/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.client.collector;

import java.lang.management.ManagementFactory;
import java.lang.management.ThreadMXBean;
import java.util.ArrayList;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.appspy.client.common.Collector;
import org.appspy.client.common.CollectorFactory;
import org.appspy.client.common.CollectorInfo;
import org.appspy.client.common.config.CollectorConfig;
import org.appspy.core.data.CollectedData;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanFactoryAware;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.context.support.AbstractApplicationContext;
import org.springframework.core.task.TaskExecutor;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class CollectorImpl implements Collector, BeanFactoryAware
{

	protected static Log sLog = LogFactory.getLog(CollectorImpl.class);

	protected ArrayList<CollectedData> mCollectedDatas = new ArrayList<CollectedData>();

	protected CollectorConfig mConfig = null;
	protected TaskExecutor mTaskExecutor = null;

	protected ThreadLocal<CollectedData> mCollectedDataThreadLocal = new ThreadLocal<CollectedData>();

	protected BeanFactory mBeanFactory = null;

	private class CollectorTask implements Runnable
	{
		protected CollectedData mCollectedData = null;

		public CollectorTask(CollectedData collectedData)
		{
			mCollectedData = collectedData;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Runnable#run()
		 */
		public void run()
		{
			if (sLog.isInfoEnabled())
			{
				sLog.info("Collecting data : " + mCollectedData);
			}
			synchronized (mCollectedDatas)
			{
				CollectorFactory.getCollectorInfo().hit();
				if (mCollectedDatas.size() < mConfig.getMaxSamples())
				{
					CollectorFactory.getCollectorInfo().collect();
					mCollectedDatas.add(mCollectedData);
				}
			}
		}

	}

	public CollectedData beginCollection(CollectedData collectedData)
	{
		CollectedData parent = mCollectedDataThreadLocal.get();
		if (parent != null)
		{
			parent.addChild(collectedData);
		}
		mCollectedDataThreadLocal.set(collectedData);

		if (sLog.isDebugEnabled())
		{
			sLog.debug("Starting data collection : " + collectedData);
		}

		return collectedData;
	}

	public void endCollection()
	{
		CollectedData collectedData = mCollectedDataThreadLocal.get();
		CollectedData parent = collectedData.getParent();
		mCollectedDataThreadLocal.set(parent);

		if (sLog.isDebugEnabled())
		{
			sLog.debug("Finishing data collection : " + collectedData);
		}

		if (parent == null)
		{
			send(collectedData);
		}
	}

	/**
	 * @param taskExecutor
	 *            the taskExecutor to set
	 */
	public void setTaskExecutor(TaskExecutor taskExecutor)
	{
		mTaskExecutor = taskExecutor;
	}

	protected void send(CollectedData collectedData)
	{
		mTaskExecutor.execute(new CollectorTask(collectedData));
	}

	protected ArrayList<CollectedData> getCollectedDatas(boolean reset)
	{
		ArrayList<CollectedData> collectedDatas = mCollectedDatas;

		if (reset)
		{
			mCollectedDatas = new ArrayList<CollectedData>();
		}

		return collectedDatas;
	}

	public void init(CollectorConfig config)
	{
		mConfig = config;

		ThreadMXBean mxb = ManagementFactory.getThreadMXBean();
		mxb.setThreadCpuTimeEnabled(mConfig.getCpuTimeEnabled());
		mxb.setThreadContentionMonitoringEnabled(mConfig.getThreadContentionEnabled());

		if (CollectorFactory.getCollectorInfo().getStatus() == CollectorInfo.STATUS_STOPPED)
		{
			CollectorFactory.getCollectorInfo().setStatus(CollectorInfo.STATUS_RUNNING, null);
		}
	}

	public void destroy()
	{
		if (mBeanFactory instanceof AbstractApplicationContext)
		{
			// Is this case possible ? Anyway ...
			AbstractApplicationContext appCtx = (AbstractApplicationContext) mBeanFactory;
			appCtx.close();
		} else if (mBeanFactory instanceof DefaultListableBeanFactory)
		{
			DefaultListableBeanFactory appCtx = (DefaultListableBeanFactory) mBeanFactory;
			appCtx.destroySingletons();
		}
	}

	public void setBeanFactory(BeanFactory beanFactory) throws BeansException
	{
		mBeanFactory = beanFactory;
	}

}
